# U3 and U4 are conjugated by this element in U2 (which
# exchanges the two plane classes, and preserves lines).
g := [
	[0,0,0,1,0,0],
	[0,0,0,0,0,1],
	[0,0,0,0,1,0],
	[1,0,0,0,0,0],
	[0,0,1,0,0,0],
	[0,1,0,0,0,0]]*z^0;

#
# Compute the stabilizer intersections
#
Display("Computing the double stabilizers...");
U12 := Stabilizer(U2, u_space[1], OnSubspacesByCanonicalBasis);

U13 := Stabilizer(U3, u_space[1], OnSubspacesByCanonicalBasis);
U23 := Stabilizer(U3, u_space[2], OnSubspacesByCanonicalBasis);
U123 := Stabilizer(U23, u_space[1], OnSubspacesByCanonicalBasis);

U14 := Stabilizer(U4, u_space[1], OnSubspacesByCanonicalBasis);
U24 := Stabilizer(U4, u_space[2], OnSubspacesByCanonicalBasis);
U124 := Stabilizer(U24, u_space[1], OnSubspacesByCanonicalBasis);


Display("Finding generators suitable for amalgamation...");

repeat
	x := PseudoRandom(U12);
until ClosureGroup(U123,x)=U12;

repeat
	y := PseudoRandom(U23);
until ClosureGroup(U123,y)=U23;

repeat
	a := PseudoRandom(U13);
until ClosureGroup(U123,a)=U13 and ClosureGroup(U23,a)=U3;

repeat
	c := PseudoRandom(U14);
until ClosureGroup(U124,c)=U14;

repeat
	h := PseudoRandom(U124)^(g^-1);
until U124 = ClosureGroup(U123^g,h^g);


# Verify "nice" generator properties -- namely, that the intersections of
# the groups are generated by the intersections of their generating sets.
Display("Verifying...");

Assert(0, U12 = ClosureGroup(U123,x));
Assert(0, U23 = ClosureGroup(U123,y));

Assert(0, U12 = ClosureGroup(U124,x));
Assert(0, U24 = ClosureGroup(U124,y^g));

Assert(0, Size(U1) = SizeViaOrbit(ClosureGroup(U123,[x,a]),v_orb));
Assert(0, U2 = ClosureGroup(U123,[x,y,y^g]));
Assert(0, U3 = ClosureGroup(U123,[y,a]));
Assert(0, U4 = ClosureGroup(U124,[y^g,a^g]));


Display("Building the amalgam...");

g123 := ShallowCopy(SmallGeneratingSet(U123));
g123_names := List([1..Length(g123)], n -> Concatenation("f", String(n)));
g124_names := List(g123_names, x -> Concatenation(x, "G"));

gens := Concatenation(
			g123,
			List(g123, e->e^g),
			[h,h^g,x,y,y^g,a,a^g,c,c^g,g]);
gen_names := Concatenation(
			g123_names,
			g124_names,
			["h", "hG", "x", "y", "yG", "a", "aG", "c", "cG", "g"]);

Assert(0, Size(gens) = Size(gen_names));

u_gens := [];
u_gens[1] := Concatenation(g123_names, g124_names, ["hG","x","a", "c"]);
u_gens[2] := Concatenation(g123_names, g124_names, ["h","hG","x","y","yG","g"]);
u_gens[3] := Concatenation(g123_names,             ["h","y","a", "cG"]);
u_gens[4] := Concatenation(            g124_names, ["hG","yG","aG", "c"]);

# We provided lists of generators identified with elements in G2. Now
# we compute all relators between these.
Read("build_rels.gap");

# Identify generators with their conjugates (h/hG, y/yG, a/aG, c/cG), by adding
# suitable relators. For this it is sufficient to add the relations between
# a / a^g, and c / c^g, since the others are contained in U2 together with g,
# so the corresponding relators are already present.

fg_gens := GeneratorsOfGroup(FG);
g_in_FG := fg_gens[Position(gens, g)];
for e in ["a", "c"] do
	tmp1 := fg_gens[Position(gen_names, e)];
	tmp2 := fg_gens[Position(gen_names, Concatenation(e, "G"))];
	Add(rels, tmp2^-1 * tmp1^g_in_FG );
od;
